require "Window"
require "ApolloColor"

local LUI_AugmentorHelper = {}
local GeminiColor = Apollo.GetPackage("GeminiColor").tPackage

function LUI_AugmentorHelper:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    self.locked = false
    self.settings = {
    	config = false,
    	showIndicator = false,
    	showBackup = false,
    	backupPos = {
    		left = -215,
			top = 10,
			right = 215,
			bottom = 50,
    	},
    	posX = -70,
    	posY = -70,
    	length = 80,
    	offset = 20,
    	height = 15,
    	thickness = 15,
    	spacing = 5,
    	mode = "static",
    	["North"] = {
    		line = true,
    		timer = true,
    		sound = false,
    		color = "fffff400",
    		label = "North",
    	},
    	["West"] = {
    		line = true,
    		timer = true,
    		sound = false,
    		color = "ff05ff00",
    		label = "West",
    	},
    	["East"] = {
    		line = true,
    		timer = true,
    		sound = false,
    		color = "ff00bfff",
    		label = "East",
    	},
    	["Backup"] = {
    		line = true,
    		timer = true,
    		sound = false,
    		color = "ffffffff",
    		label = "Backup",
    	},
	}
    return o
end

function LUI_AugmentorHelper:Init()
	Apollo.RegisterAddon(self, true, "LUI AugmentorHelper", nil)
end

function LUI_AugmentorHelper:OnLoad()
	Apollo.LoadSprites("sprites.xml")
	self.xmlDoc = XmlDoc.CreateFromFile("LUI_AugmentorHelper.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

function LUI_AugmentorHelper:OnDocLoaded()
	if self.xmlDoc == nil then
		return
	end

	Apollo.RegisterEventHandler("UnitEnteredCombat", "OnEnteredCombat", self)
	Apollo.RegisterEventHandler("UnitCreated", "OnUnitCreated", self)
	Apollo.RegisterEventHandler("NextFrame", "OnUpdate", self)
	Apollo.RegisterEventHandler("ChangeWorld", "OnChangeZone", self)
    Apollo.RegisterEventHandler("SubZoneChanged", "OnChangeZone", self)
    Apollo.RegisterTimerHandler("CheckZoneTimer", "OnChangeZone", self)
    Apollo.RegisterTimerHandler("CheckCombatTimer", "OnCheckCombat", self)

	self.screen = Apollo.LoadForm(self.xmlDoc, "Overlay", "InWorldHudStratum", self)
	self.wndSettings = Apollo.LoadForm(self.xmlDoc, "Settings", nil, self)
	self.wndSettings:Show(false,true)

	self.wndTimer = Apollo.LoadForm(self.xmlDoc, "Timer", nil, self)
	self.wndTimer:FindChild("Lock"):Show(false,true)
	self.wndTimer:Show(false,true)

	self.wndBackup = Apollo.LoadForm(self.xmlDoc, "Backup", nil, self)
	self.wndBackup:FindChild("Lock"):Show(false,true)
	self.wndBackup:Show(false,true)

	self:BuildTimer()
	self:CheckZone()
	self:LoadSettings()
end

function LUI_AugmentorHelper:BuildTimer()
	self.wndTimer:SetAnchorOffsets(
		self.settings.posX,
		self.settings.posY,
		self.settings.posX + 140,
		self.settings.posY + 140
	)

	self.wndBackup:SetAnchorOffsets(
		self.settings.backupPos.left,
		self.settings.backupPos.top,
		self.settings.backupPos.right,
		self.settings.backupPos.bottom
	)

	for id,timer in pairs(self.wndTimer:GetChildren()) do
		if timer:GetName() ~= "Lock" then
			timer:Show(false,true)
			timer:FindChild("Label"):SetText(self.settings[timer:GetName()].label)
			timer:FindChild("BG"):SetBGColor("33000000")
			timer:FindChild("BarBG"):SetSprite("agh_circle")
			timer:FindChild("Bar"):SetFullSprite("agh_circle")
			timer:FindChild("Bar"):SetMax(100000)
			timer:FindChild("Bar"):SetStyleEx("Clockwise", false)
			timer:FindChild("Bar"):SetBGColor(self.settings[timer:GetName()].color)
		end
	end

	for id,backup in pairs(self.wndBackup:GetChildren()) do
		if backup:GetName() ~= "Lock" then
			backup:SetSprite("agh_button")
			backup:FindChild("Button"):SetText(self.settings[backup:GetName()].label)
			backup:FindChild("BG"):SetBGColor(self.settings[backup:GetName()].color)
		end
	end
end

function LUI_AugmentorHelper:OnUpdate()
	if not self.unitPlayer then 
		self.unitPlayer = GameLib.GetPlayerUnit()
		return 
	end 

	if self.locked == true then
		return
	end

	if not self.lastCheck then
		self.lastCheck = Apollo.GetTickCount()
	end

	if (Apollo.GetTickCount() - self.lastCheck) > 10 then
		self.screen:DestroyAllPixies()

		if self.settings.config == true then
			self:DrawLines(self.unitPlayer)
			self.lastCheck = Apollo.GetTickCount()
			return
		end

		if not self.run or not self.run == true then
			return
		end

		if not self.bInCombat == true then
			return
		end

		if not self.laser then 
			return
		end

		self:DrawLines(self.laser)
		self.lastCheck = Apollo.GetTickCount()
	end
end

function LUI_AugmentorHelper:DrawLines(unit)
	if not unit then
		return
	end

	local tface = unit:GetFacing()

	if not tface then
		return
	end

	local targ_screen_pos = GameLib.GetUnitScreenPosition(unit)

	if not targ_screen_pos then
		return
	end

	if not self.boss then
		self.boss = math.atan2(tface.x, tface.z) + math.rad(180)
		self.angle = math.deg(math.atan2(tface.x, tface.z))

		if self.angle < 0 then
			self.angle = 360 + self.angle
		end
	end

	if not self.speed then
		if self.boss ~= nil and self.angle ~= nil then
			if not self.speedtimer then
				local angle = math.deg(math.atan2(tface.x, tface.z))

				if angle < 0 then
					angle = 360 + angle
				end
		
				if angle ~= self.angle then
					self.speedtimer = Apollo.GetTickCount()
				end
			else
				local diff = Apollo.GetTickCount() - self.speedtimer
				local angle = math.deg(math.atan2(tface.x, tface.z))

				if angle < 0 then
					angle = 360 + angle
				end

				if diff > 1000 then
					self.speed = tonumber(angle - self.angle)
				end
			end
		end
	end

	if self.speed ~= nil and self.boss ~= nil then
		self:CreateTimer(tface)
	else
		self.wndTimer:FindChild("North"):FindChild("Bar"):SetProgress(0)
		self.wndTimer:FindChild("North"):Show(false,true)

		self.wndTimer:FindChild("West"):FindChild("Bar"):SetProgress(0)
		self.wndTimer:FindChild("West"):Show(false,true)

		self.wndTimer:FindChild("East"):FindChild("Bar"):SetProgress(0)
		self.wndTimer:FindChild("East"):Show(false,true)

		self.wndTimer:FindChild("Backup"):FindChild("Bar"):SetProgress(0)
		self.wndTimer:FindChild("Backup"):Show(false,true)
	end

	if self.timer ~= nil then
		if self.timer["North"] < self.timer["West"] and self.timer["North"] < self.timer["East"] then
			self.next = "North"
		elseif self.timer["West"] < self.timer["North"] and self.timer["West"] < self.timer["East"] then
			self.next = "West"
		elseif self.timer["East"] < self.timer["North"] and self.timer["East"] < self.timer["West"] then
			self.next = "East"
		end

		self.wndBackup:Show((self.settings.showBackup == true and not self.backup),true)

		for id,_ in pairs(self.timer) do
			self.wndTimer:FindChild(id):Show((id == self.next and self.settings[id].timer == true),true)
		end

		if self.backup and self.backup == self.next and self.settings["Backup"].timer == true then
			self.wndTimer:FindChild(self.next):Show(false,true)
			self.wndTimer:FindChild("Backup"):Show(true,true)

			self.wndTimer:FindChild("Backup"):FindChild("Text"):SetText(self.wndTimer:FindChild(self.backup):FindChild("Text"):GetText())
			self.wndTimer:FindChild("Backup"):FindChild("Bar"):SetProgress(self.wndTimer:FindChild(self.backup):FindChild("Bar"):GetProgress())
			self.wndTimer:FindChild("Backup"):FindChild("Bar"):SetBGColor(self.settings[self.backup].color)

			if self.settings["Backup"].sound == true and self.settings[self.backup].sound == false then
				self:PlaySound("Backup",tonumber(self.wndTimer:FindChild(self.backup):FindChild("Text"):GetText()))
			end
		else
			self.wndTimer:FindChild("Backup"):Show(false,true)
		end
	end

	if self.next then
		self:CreateLines(unit)
	end
end

function LUI_AugmentorHelper:CreateLines(unit)
	if not unit then 
		return
	end 

	local tface = unit:GetFacing()

	if not tface then 
		return 
	end 

	local north = math.atan2(tface.x, tface.z) + math.rad(self.settings.offset)
	local east = north + math.rad(120)
	local west = north + math.rad(-120)
	local sizeMod = 1

	local tpos = unit:GetPosition()
	local tposV = Vector3.New(tpos.x,tpos.y+(self.settings.height / 10),tpos.z)

	if (self.settings[self.next].line == true or (self.backup and self.backup == self.next and self.settings["Backup"].line == true)) then
		local lineColor = self.next and self.settings[self.next].color or self.settings["East"].color

		local frontEnd = Vector3.New(tposV.x+self.settings.length*math.sin(north), tposV.y, tposV.z+self.settings.length*math.cos(north))
		local frontOffset = Vector3.InterpolateLinear(frontEnd, tposV, (1 - (self.settings.spacing / 100)))
		self:DrawLine(frontEnd, frontOffset, lineColor,sizeMod)

		if self.settings.mode == "static" then
			local leftEnd = Vector3.New(tposV.x+self.settings.length*math.sin(east), tposV.y, tposV.z+self.settings.length*math.cos(east))
			local leftOffset = Vector3.InterpolateLinear(leftEnd, tposV, (1 - (self.settings.spacing / 100)))
			self:DrawLine(leftEnd, leftOffset, lineColor,sizeMod)

			local rightEnd = Vector3.New(tposV.x+self.settings.length*math.sin(west), tposV.y, tposV.z+self.settings.length*math.cos(west))
			local rightOffset = Vector3.InterpolateLinear(rightEnd, tposV, (1 - (self.settings.spacing / 100)))
			self:DrawLine(rightEnd, rightOffset, lineColor,sizeMod)
		end
	end

	if self.settings.showIndicator == true then
		if self.settings.mode == "static" then
			local bossNorthPos = self.boss
			local bossEastPos = self.boss + math.rad(-120)
			local bossWestPos = self.boss + math.rad(120)

			local bossEnd = Vector3.New(tposV.x+self.settings.length*math.sin(bossNorthPos), tposV.y, tposV.z+self.settings.length*math.cos(bossNorthPos))
			local bossOffset = Vector3.InterpolateLinear(bossEnd, tposV, (1 - (self.settings.spacing / 100)))
			self:DrawLine(bossEnd, bossOffset, "ffff0000",sizeMod)

			bossEnd = Vector3.New(tposV.x+self.settings.length*math.sin(bossEastPos), tposV.y, tposV.z+self.settings.length*math.cos(bossEastPos))
			bossOffset = Vector3.InterpolateLinear(bossEnd, tposV, (1 - (self.settings.spacing / 100)))
			self:DrawLine(bossEnd, bossOffset, "ffff0000",sizeMod)

			bossEnd = Vector3.New(tposV.x+self.settings.length*math.sin(bossWestPos), tposV.y, tposV.z+self.settings.length*math.cos(bossWestPos))
			bossOffset = Vector3.InterpolateLinear(bossEnd, tposV, (1 - (self.settings.spacing / 100)))
			self:DrawLine(bossEnd, bossOffset, "ffff0000",sizeMod)
		else
			local pos = self.boss

			if self.next == "East" then
				pos = self.boss + math.rad(-120)
			elseif self.next == "West" then
				pos = self.boss + math.rad(120)
			end

			local bossEnd = Vector3.New(tposV.x+self.settings.length*math.sin(pos), tposV.y, tposV.z+self.settings.length*math.cos(pos))
			local bossOffset = Vector3.InterpolateLinear(bossEnd, tposV, (1 - (self.settings.spacing / 100)))
			self:DrawLine(bossEnd, bossOffset, "ffff0000",sizeMod)
		end
	end
end

function LUI_AugmentorHelper:CreateTimer(tface)
	local tangle = math.atan2(tface.x, tface.z) + math.rad(self.settings.offset)
	local boss = tonumber(math.deg(self.boss))

	if boss < 0 then
		boss = 360 + boss
	end

	self.wndTimer:Show(true,true)

	self.lasers = {
		["North"] = tangle,
		["East"] = tangle + math.rad(120),
		["West"] = tangle + math.rad(-120)
	}

	self.timer = {
		["North"] = 0,
		["East"] = 0,
		["West"] = 0,
	}

	for id,radius in pairs(self.lasers) do
		local degree = tonumber(math.deg(radius))

		if degree < 0 then
			degree = 360 + degree
		end

		local missing = 0

		if degree > boss then
			missing = 360 - degree + boss
		else
			missing = boss - degree
		end

		local total = (360 / tonumber(self.speed)) / 3
		local remaining = missing / tonumber(self.speed)
		local percent = 100000 - ((remaining * 100000) / total)

		self.timer[id] = remaining

		if remaining > 0 then
			if remaining < 10 then
				remaining = Apollo.FormatNumber(remaining, 1, true)
			else
				remaining = Apollo.FormatNumber(remaining, 0, true)
			end

			self.wndTimer:FindChild(id):FindChild("Text"):SetText(remaining)
			self.wndTimer:FindChild(id):FindChild("Bar"):SetProgress(percent)

			self:PlaySound(id,remaining)
		else
			self.wndTimer:FindChild(id):FindChild("Text"):SetText("")
			self.wndTimer:FindChild(id):FindChild("Bar"):SetProgress(0)
		end
	end
end

function LUI_AugmentorHelper:PlaySound(id,remaining)
	if not id or not remaining then
		return false
	end

	if self.settings[id].sound == true then
		if not self.played then
			self.played = {}
		end

		if not self.played[id] then
			self.played[id] = 5.2
		end

		if self.played[id] == 0.5 and tonumber(remaining) > 6 then
			self.played[id] = 5.2
		end

		if tonumber(remaining) < tonumber(self.played[id] - 1) and tonumber(self.played[id] - 1) > 0 then
			if self.played[id] == 5.2 then
				self.played[id] = 4.5
			end
			Sound.PlayFile("Sounds\\"..tostring(math.floor(self.played[id] - 1))..".wav")
			self.played[id] = self.played[id] - 1
		end
	end
end

function LUI_AugmentorHelper:Reset()
	self.wndTimer:FindChild("North"):FindChild("Bar"):SetProgress(0)
	self.wndTimer:FindChild("North"):Show(false,true)

	self.wndTimer:FindChild("West"):FindChild("Bar"):SetProgress(0)
	self.wndTimer:FindChild("West"):Show(false,true)

	self.wndTimer:FindChild("East"):FindChild("Bar"):SetProgress(0)
	self.wndTimer:FindChild("East"):Show(false,true)

	self.wndTimer:FindChild("Backup"):FindChild("Bar"):SetProgress(0)
	self.wndTimer:FindChild("Backup"):Show(false,true)

	self.wndBackup:Show(false,true)
	self.screen:DestroyAllPixies()

	self.laser = nil
	self.boss = nil
	self.angle = nil
	self.speed = nil
	self.speedtimer = nil
	self.played = nil
	self.timer = nil
	self.next = nil
	self.backup = nil
end

function LUI_AugmentorHelper:DrawLine(startV, endV, col, sizeMod)
	local startPoint = GameLib.WorldLocToScreenPoint(startV)
	local endPoint = GameLib.WorldLocToScreenPoint(endV)

	self.screen:AddPixie({
		bLine = true,
		fWidth = self.settings.thickness*sizeMod,
		cr = col,
		loc = {
			fPoints = { 0, 0, 0, 0 },
			nOffsets = { startPoint.x, startPoint.y, endPoint.x, endPoint.y }
		}
	})
end

function LUI_AugmentorHelper:OnBackup(wndHandler,wndControl)
	self.backup = wndControl:GetParent():GetName()
	self.wndBackup:Show(false,true)
end

function LUI_AugmentorHelper:CheckZone()
	if not self.zone then
		self:OnChangeZone()
		return
	end

	self.run = (self.zone.continentId == 91 and self.zone.parentZoneId == 0)
end

function LUI_AugmentorHelper:OnChangeZone()
	local zone = GameLib.GetCurrentZoneMap()

	if zone then
		self.zone = {
			strName = zone.strName,
			continentId = zone.continentId,
			parentZoneId = zone.parentZoneId,
			id = zone.id,
		}
		self:CheckZone()
	else
		Apollo.CreateTimer("CheckZoneTimer", 3, false)
	end
end

function LUI_AugmentorHelper:OnEnteredCombat(unit, bInCombat)
	if not self.run or not self.run == true then
		return
	end

	if unit and unit:IsValid() then 
		if unit:IsThePlayer() == true then
			if bInCombat == true then 
				self.bInCombat = true
			else
				for i, aura in pairs(unit:GetBuffs().arBeneficial) do
					if aura.splEffect:GetName() == "Void Slip" or aura.splEffect:GetName() == "Homeward Bound" then
						Apollo.CreateTimer("CheckCombatTimer", 10, false)
						return
					end
				end

				self.bInCombat = false
				self:Reset()
			end
		end
	end
end

function LUI_AugmentorHelper:OnUnitCreated(unit)
	if not self.run or not self.run == true then
		return
	end

	if unit and unit:IsValid() then 
		if unit:GetName() == "Organic Incinerator" then
			if not self.laser then
	        	self.laser = unit
	        else 
	        	if not self.bInCombat == true then
	        		if self.laser:GetId() ~= unit:GetId() then 
	        			self.laser = unit
	        		end 
	        	end
	        end
	    end
	end
end

function LUI_AugmentorHelper:OnCheckCombat()
	if not GameLib.GetPlayerUnit():IsInCombat() then
		self:Reset()
	end
end

function LUI_AugmentorHelper:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Account then
		return
	end

	return self.settings
end

function LUI_AugmentorHelper:OnRestore(eLevel, tSavedData)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Account then
		return
	end

	if tSavedData ~= nil and tSavedData ~= "" then
        self.settings = self:Extend(self.settings, tSavedData)
    end
end

function LUI_AugmentorHelper:OnConfigure()
	self:OnToggleMenu()
end

function LUI_AugmentorHelper:OnToggleMenu()
	if self.wndSettings then
		if self.wndSettings:IsShown() then
			self.wndSettings:Close()
		else
			self.wndSettings:Invoke()
		end
	end
end

function LUI_AugmentorHelper:LoadSettings()
	-- Donation
    self.wndSettings:FindChild("DonateForm"):Show(false,true)
	self.wndSettings:FindChild("DonateSeperator"):Show(false,true)
	self.wndSettings:FindChild("DonateBtn"):Show((GameLib.GetRealmName() == "Jabbit"),true)
	self.wndSettings:FindChild("DonateForm"):FindChild("DonateSendBtn"):Enable(false)

	-- Config Mode
	self.wndSettings:FindChild("ConfigCheckbox"):SetCheck(self.settings.config)
	self.wndSettings:FindChild("ConfigCheckbox"):SetData("config")

	-- Next Boss Indicator
	self.wndSettings:FindChild("IndicatorCheckbox"):SetCheck(self.settings.showIndicator)
	self.wndSettings:FindChild("IndicatorCheckbox"):SetData("showIndicator")

	-- Backup Checkbox
	self.wndSettings:FindChild("BackupCheckbox"):SetCheck(self.settings.showBackup)
	self.wndSettings:FindChild("BackupCheckbox"):SetData("showBackup")

	-- Addon Mode
	self.wndSettings:FindChild("ModeDropdown"):AttachWindow(self.wndSettings:FindChild("ModeDropdown"):FindChild("ChoiceContainer"))
	self.wndSettings:FindChild("ModeDropdown"):FindChild("ChoiceContainer"):Show(false)
	self.wndSettings:FindChild("ModeDropdown"):SetText("Choose")
	self.wndSettings:FindChild("ModeDropdown"):SetData("mode")

	for _,button in pairs(self.wndSettings:FindChild("ModeDropdown"):FindChild("ChoiceContainer"):GetChildren()) do
		if button:GetName() == self.settings.mode then
			button:SetCheck(true)
			self.wndSettings:FindChild("ModeDropdown"):SetText(button:GetText())
		else
			button:SetCheck(false)
		end
	end

	-- Length
	self.wndSettings:FindChild("LengthSetting"):FindChild("SliderText"):SetText(self.settings.length or 0)
	self.wndSettings:FindChild("LengthSetting"):FindChild("Slider"):SetValue(self.settings.length or 0)
	self.wndSettings:FindChild("LengthSetting"):FindChild("Slider"):SetData("length")

	-- Height
	self.wndSettings:FindChild("HeightSetting"):FindChild("SliderText"):SetText(self.settings.height or 0)
	self.wndSettings:FindChild("HeightSetting"):FindChild("Slider"):SetValue(self.settings.height or 0)
	self.wndSettings:FindChild("HeightSetting"):FindChild("Slider"):SetData("height")

	-- Thickness
	self.wndSettings:FindChild("ThicknessSetting"):FindChild("SliderText"):SetText(self.settings.thickness or 0)
	self.wndSettings:FindChild("ThicknessSetting"):FindChild("Slider"):SetValue(self.settings.thickness or 0)
	self.wndSettings:FindChild("ThicknessSetting"):FindChild("Slider"):SetData("thickness")

	-- Spacing
	self.wndSettings:FindChild("SpacingSetting"):FindChild("SliderText"):SetText(self.settings.spacing or 0)
	self.wndSettings:FindChild("SpacingSetting"):FindChild("Slider"):SetValue(self.settings.spacing or 0)
	self.wndSettings:FindChild("SpacingSetting"):FindChild("Slider"):SetData("spacing")

	-- Offset
	self.wndSettings:FindChild("OffsetSetting"):FindChild("SliderText"):SetText(self.settings.offset or 0)
	self.wndSettings:FindChild("OffsetSetting"):FindChild("Slider"):SetValue(self.settings.offset or 0)
	self.wndSettings:FindChild("OffsetSetting"):FindChild("Slider"):SetData("offset")

	-- Line Labels
	self.wndSettings:FindChild("North"):FindChild("Text"):SetText(self.settings["North"].label)
	self.wndSettings:FindChild("North"):FindChild("Text"):SetData({"North","label"})

	self.wndSettings:FindChild("West"):FindChild("Text"):SetText(self.settings["West"].label)
	self.wndSettings:FindChild("West"):FindChild("Text"):SetData({"West","label"})

	self.wndSettings:FindChild("East"):FindChild("Text"):SetText(self.settings["East"].label)
	self.wndSettings:FindChild("East"):FindChild("Text"):SetData({"East","label"})

	self.wndSettings:FindChild("Backup"):FindChild("Text"):SetText(self.settings["Backup"].label)
	self.wndSettings:FindChild("Backup"):FindChild("Text"):SetData({"Backup","label"})

	-- Line Colors
	self.wndSettings:FindChild("North"):FindChild("ColorBtn"):SetData({"North","color"})
	self.wndSettings:FindChild("North"):FindChild("ColorBackground"):SetBGColor(self.settings["North"].color)

	self.wndSettings:FindChild("West"):FindChild("ColorBtn"):SetData({"West","color"})
	self.wndSettings:FindChild("West"):FindChild("ColorBackground"):SetBGColor(self.settings["West"].color)

	self.wndSettings:FindChild("East"):FindChild("ColorBtn"):SetData({"East","color"})
	self.wndSettings:FindChild("East"):FindChild("ColorBackground"):SetBGColor(self.settings["East"].color)

	-- Play Sound
	self.wndSettings:FindChild("North"):FindChild("SoundCheckbox"):SetCheck(self.settings["North"].sound)
	self.wndSettings:FindChild("North"):FindChild("SoundCheckbox"):SetData({"North","sound"})

	self.wndSettings:FindChild("West"):FindChild("SoundCheckbox"):SetCheck(self.settings["West"].sound)
	self.wndSettings:FindChild("West"):FindChild("SoundCheckbox"):SetData({"West","sound"})

	self.wndSettings:FindChild("East"):FindChild("SoundCheckbox"):SetCheck(self.settings["East"].sound)
	self.wndSettings:FindChild("East"):FindChild("SoundCheckbox"):SetData({"East","sound"})

	self.wndSettings:FindChild("Backup"):FindChild("SoundCheckbox"):SetCheck(self.settings["Backup"].sound)
	self.wndSettings:FindChild("Backup"):FindChild("SoundCheckbox"):SetData({"Backup","sound"})

	-- Show Timer
	self.wndSettings:FindChild("North"):FindChild("TimerCheckbox"):SetCheck(self.settings["North"].timer)
	self.wndSettings:FindChild("North"):FindChild("TimerCheckbox"):SetData({"North","timer"})

	self.wndSettings:FindChild("West"):FindChild("TimerCheckbox"):SetCheck(self.settings["West"].timer)
	self.wndSettings:FindChild("West"):FindChild("TimerCheckbox"):SetData({"West","timer"})

	self.wndSettings:FindChild("East"):FindChild("TimerCheckbox"):SetCheck(self.settings["East"].timer)
	self.wndSettings:FindChild("East"):FindChild("TimerCheckbox"):SetData({"East","timer"})

	self.wndSettings:FindChild("Backup"):FindChild("TimerCheckbox"):SetCheck(self.settings["Backup"].timer)
	self.wndSettings:FindChild("Backup"):FindChild("TimerCheckbox"):SetData({"Backup","timer"})

	-- Show Line
	self.wndSettings:FindChild("North"):FindChild("LineCheckbox"):SetCheck(self.settings["North"].line)
	self.wndSettings:FindChild("North"):FindChild("LineCheckbox"):SetData({"North","line"})

	self.wndSettings:FindChild("West"):FindChild("LineCheckbox"):SetCheck(self.settings["West"].line)
	self.wndSettings:FindChild("West"):FindChild("LineCheckbox"):SetData({"West","line"})

	self.wndSettings:FindChild("East"):FindChild("LineCheckbox"):SetCheck(self.settings["East"].line)
	self.wndSettings:FindChild("East"):FindChild("LineCheckbox"):SetData({"East","line"})

	self.wndSettings:FindChild("Backup"):FindChild("LineCheckbox"):SetCheck(self.settings["Backup"].line)
	self.wndSettings:FindChild("Backup"):FindChild("LineCheckbox"):SetData({"Backup","line"})
end

function LUI_AugmentorHelper:OnTextChange(wndHandler, wndControl)
	local setting = wndControl:GetData()
	local value = wndHandler:GetText()

	if type(setting) == "string" then
		self.settings[setting] = value
	else
		self.settings[setting[1]][setting[2]] = value
	end

	self:BuildTimer()
end

function LUI_AugmentorHelper:OnDropdownToggle(wndHandler, wndControl)
	if wndHandler ~= wndControl then
		return
	end

	wndControl:FindChild("ChoiceContainer"):Show(wndControl:IsChecked())
end

function LUI_AugmentorHelper:OnDropdownChoose(wndHandler, wndControl)
	local dropdown = wndControl:GetParent():GetParent()
	local setting = dropdown:GetData()
	local value = wndControl:GetName()

	dropdown:SetText(wndControl:GetText())
	wndControl:GetParent():Close()

	if type(setting) == "string" then
		self.settings[setting] = value
	else
		self.settings[setting[1]][setting[2]] = value
	end
end

function LUI_AugmentorHelper:OnSliderChange(wndHandler, wndControl)
	local setting = wndControl:GetData()
	local value = wndHandler:GetValue()

	wndControl:GetParent():FindChild("SliderText"):SetText(value)

	if type(setting) == "string" then
		self.settings[setting] = value
	else
		self.settings[setting[1]][setting[2]] = value
	end
end

function LUI_AugmentorHelper:OnCheckbox(wndHandler, wndControl)
	local setting = wndControl:GetData()
	local value = wndHandler:IsChecked()

	if type(setting) == "string" then
		self.settings[setting] = value
	else
		self.settings[setting[1]][setting[2]] = value
	end

	self:BuildTimer()

	if setting == "config" then
		self:Reset()
	end
end

function LUI_AugmentorHelper:OnUnlock(wndHandler, wndControl)
	self.locked = wndHandler:IsChecked()

	self.wndTimer:Show(wndHandler:IsChecked(),true)
	self.wndTimer:SetStyle("Moveable", wndHandler:IsChecked())
	self.wndTimer:SetStyle("IgnoreMouse", not wndHandler:IsChecked())
	self.wndTimer:FindChild("Lock"):Show(wndHandler:IsChecked(),true)

	self.wndTimer:FindChild("North"):Show(not wndHandler:IsChecked())
	self.wndTimer:FindChild("West"):Show(not wndHandler:IsChecked())
	self.wndTimer:FindChild("East"):Show(not wndHandler:IsChecked())
	self.wndTimer:FindChild("Backup"):Show(not wndHandler:IsChecked())

	self.wndBackup:Show(wndHandler:IsChecked(),true)
	self.wndBackup:SetStyle("Sizable", wndHandler:IsChecked())
	self.wndBackup:SetStyle("Moveable", wndHandler:IsChecked())
	self.wndBackup:SetStyle("IgnoreMouse", not wndHandler:IsChecked())
	self.wndBackup:FindChild("Lock"):Show(wndHandler:IsChecked(),true)

	self.wndBackup:FindChild("North"):Show(not wndHandler:IsChecked())
	self.wndBackup:FindChild("West"):Show(not wndHandler:IsChecked())
	self.wndBackup:FindChild("East"):Show(not wndHandler:IsChecked())
end

function LUI_AugmentorHelper:OnMove(wndHandler, wndControl)
	local nLeft, nTop, nRight, nBottom = wndControl:GetAnchorOffsets()

	if wndControl:GetName() == "Timer" then
		self.settings.posX = nLeft
		self.settings.posY = nTop
	end

	if wndControl:GetName() == "Backup" then
		self.settings.backupPos = {
			left = nLeft,
			top = nTop,
			right = nRight,
			bottom = nBottom,
		}
	end
end

function LUI_AugmentorHelper:OnBtnChooseColor(wndHandler, wndControl)
	local setting = wndControl:GetData()

	if type(setting) == "string" then
		setting = self.settings[setting]
	else
		setting = self.settings[setting[1]][setting[2]]
	end

	GeminiColor:ShowColorPicker(self, {
		callback = "OnColorPicker",
		strInitialColor = setting,
		bCustomColor = true,
		bAlpha = false
	}, wndControl)
end

function LUI_AugmentorHelper:OnColorPicker(wndHandler, wndControl)
	if wndHandler == nil or string.len(wndHandler) ~= 8 then
		return
	end

	local setting = wndControl:GetData()
	local value = wndHandler

	wndControl:GetParent():FindChild("ColorBackground"):SetBGColor(value)

	if type(setting) == "string" then
		self.settings[setting] = value
	else
		self.settings[setting[1]][setting[2]] = value
	end

	self:BuildTimer()
end

function LUI_AugmentorHelper:OnDonate(wndHandler, wndControl)
	self.wndSettings:FindChild("DonateSeperator"):Show(wndHandler:IsChecked(),true)
    self.wndSettings:FindChild("DonateForm"):Show(wndHandler:IsChecked(),true)
end

function LUI_AugmentorHelper:OnDonationChanged(wndHandler, wndControl)
	local amount = self.wndSettings:FindChild("DonateForm"):FindChild("CashWindow"):GetAmount()
	local recipient = "Loui NaN"

	if self.unitPlayer:GetFaction() ~= 166 then
		recipient = "Loui x"
	end

	if amount > 0 then
    	self.wndSettings:FindChild("DonateForm"):FindChild("DonateSendBtn"):SetActionData(GameLib.CodeEnumConfirmButtonType.SendMail, recipient, "Jabbit", "LUI AugmentorHelper Donation", tostring(self.unitPlayer:GetName()) .. " donated something for you!", nil, MailSystemLib.MailDeliverySpeed_Instant, 0, self.wndSettings:FindChild("DonateForm"):FindChild("CashWindow"):GetCurrency())
    	self.wndSettings:FindChild("DonateForm"):FindChild("DonateSendBtn"):Enable(true)
    else
    	self.wndSettings:FindChild("DonateForm"):FindChild("DonateSendBtn"):Enable(false)
    end
end

function LUI_AugmentorHelper:OnDonationSent()
    self.wndSettings:FindChild("DonateBtn"):SetCheck(false)
    self.wndSettings:FindChild("DonateForm"):Show(false)
    self.wndSettings:FindChild("DonateSeperator"):Show(false)
    self.wndSettings:FindChild("DonateForm"):FindChild("CashWindow"):SetAmount(0)
    self.wndSettings:FindChild("DonateForm"):FindChild("DonateSendBtn"):Enable(false)
    Print("Thank you very much!!! <3")
end

function LUI_AugmentorHelper:Extend(...)
    local args = {...}
    for i = 2, #args do
        for key, value in pairs(args[i]) do
            args[1][key] = value
        end
    end
    return args[1]
end

local LUI_AugmentorHelperInst = LUI_AugmentorHelper:new()
LUI_AugmentorHelperInst:Init()
